//  Implementation of the isolation model described in the main text

#include <R.h>
#include <Rmath.h>
#include <R_ext/Rdynload.h>
#include <Rdefines.h>

//  Macros to ameliorate code lisibility

//  Parameters
#define B0 (p[parindex[0]]) //sporadic acquisition rate
#define B1 (p[parindex[1]]) //transmission rate from unisolated patients (C1 and C2)
#define B2 (p[parindex[2]]) //transmission rate from isolated patients (I)
#define D1 (p[parindex[3]]) //isolation rate for imported cases (C1)
#define D2 (p[parindex[4]]) //isolation rate for acquired cases (C2)
#define MU_S (p[parindex[5]]) //Discharge rate of susceptible patients (S)
#define MU_C (p[parindex[6]]) //Discharge rate of colonized, unisolated patients
#define MU_I (p[parindex[7]]) //Discharge rate of isolated patients
#define SIGMA (p[parindex[8]]) //Admission prevalence
#define RHO ((p[parindex[9]])) //test sensitivity
#define NI ((p[parindex[10]])) //Isolation ward capacity
#define N ((p[parindex[11]])) //Ward size
#define EPS ((p[parindex[12]])) //Fraction of colonized inpatients directly put in isolation

// State variables
#define C1 (x[stateindex[0]]) //number of unisolated patients, already colonized upon admission
#define C2 (x[stateindex[1]]) //number of unisolated patients, not already colonized upon admission
#define I (x[stateindex[2]]) //number of colonized, isolated patients
#define INC (x[stateindex[3]]) //cumulative number of acquisitions (reset to 0 at each new observation)

//  Observations
#define PV_OBS (y[obsindex[0]]) //prevalence data
#define INC_OBS (y[obsindex[1]]) //incidence data

//Observation model: binomial model with probability RHO
void dObs (double *lik, double *y, double *x, double *p, int give_log,
		int *obsindex, int *stateindex, int *parindex, int *covindex,
		int ncovars, double *covars, double t) {

	double l1 = dbinom(INC_OBS, nearbyint(INC), 1.0, 0);
	double l2 = dbinom(PV_OBS, nearbyint(C1+C2+I), RHO, 0);
	*lik= (give_log) ? log(l1 * l2) : (l1 * l2);
}

void rObs (double *y, double *x, double *p,
		int *obsindex, int *stateindex, int *parindex, int *covindex,
		int ncovars, double *covars, double t) {

	PV_OBS = rbinom(nearbyint(C1+C2+I), RHO);
	INC_OBS = rbinom(nearbyint(INC), 1.0);
}

#undef PV_OBS
#undef INC_OBS

//  Transmission model, simulated with Gillespie exact algorithm
double rSim(int j, double t, const double *x, const double *p,
		int *stateindex, int *parindex, int *covindex,
		int ncovar, double *covars){

	//  Instantaneous transition rate
	double rate=0.0;

	// check that all values are sound
	// otherwise returns without doing anything
	if (!(R_FINITE(B0)) ||
			!(R_FINITE(B1)) ||
			!(R_FINITE(B2)) ||
			!(R_FINITE(D1)) ||
			!(R_FINITE(D2)) ||
			!(R_FINITE(MU_S)) ||
			!(R_FINITE(MU_C)) ||
			!(R_FINITE(MU_I)) ||
			!(R_FINITE(SIGMA)) ||
			!(R_FINITE(RHO)) ||
			!(R_FINITE(EPS)) ||
			!(R_FINITE(NI)) ||
			!(R_FINITE(N)) ||
			!(R_FINITE(C1)) ||
			!(R_FINITE(C2)) ||
			!(R_FINITE(I)) ||
			!(R_FINITE(INC))) return;


	switch(j){
	case 1:   //Discharge S, admission C1, (S,C1):=(S-1,C1+1)
		rate = MU_S * SIGMA * (1.0 - EPS) * (N-C1-C2-I);
		break;
	case 2:   //New acquisition, (S,C2):=(S-1,C2+1)
		rate = (B0 + B1 * (C1+C2) + B2 * I) * (N-C1-C2-I);
		break;
	case 3:   //Discharge S, admission I (S,I):=(S-1,I+1)
		if(I>=NI) rate = 0.0;
		else rate = MU_S * SIGMA * EPS * (N-C1-C2-I);
		break;
	case 4:   //Discharge C1, admission S (S,C1):=(S+1,C1-1)
		rate = MU_C * (1.0 - SIGMA) * C1;
		break;
	case 5:   //Isolation or discharge of C1, admission I; (C1,I):=(C1-1,I+1)
		if((C1<0) || (C1>N) || (I>=NI) || (I<0)) rate = 0.0;
		else rate = RHO * D1 * C1 + MU_C * SIGMA * EPS * C1;
		break;
	case 6:   //Discharge C2, admission S; (S,C2):=(S+1,C2-1)
		rate = MU_C* (1.0 - SIGMA) * C2;
		break;
	case 7:  //Discharge C2, admission C1; (C1,C2):=(C1+1,C2-1)
		rate = MU_C * SIGMA * (1.0 - EPS) * C2;
		break;
	case 8:   //Discharge of C2, admission I; (C2,I):=(C2-1,I+1)
		if((C2<0) || (C2>N) || (I>=NI) || (I<0)) rate = 0.0;
		else rate = MU_C * SIGMA * EPS * C2;
		break;
	case 9:   //Isolation of C2; (C2,I, INC):=(C2-1,I+1, INC+1)
		if((C2<0) || (C2>N) || (I>=NI) || (I<0)) rate = 0.0;
		else rate = RHO * D2 * C2;
		break;
	case 10:   //Discharge I, admission S, (S,I):=(S+1,I-1)
		rate = MU_I * (1.0 - SIGMA) * I;
		break;
	case 11:   //Discharge I, admission C1, (C1,I):=(C1+1,I-1)
		rate = MU_I * SIGMA * (1.0 - EPS) * I;
		break;
	default:
		error("unrecognized rate code %d",j);
		break;
	}
	return rate;
}

#undef C1
#undef C2
#undef I
#undef INC

#undef B0
#undef B1
#undef B2
#undef D1
#undef D2
#undef MU_S
#undef MU_C
#undef MU_I
#undef SIGMA
#undef RHO
#undef NI
#undef N
#undef EPS
