# Author: Matthieu Domenech
# Profile loglike function: takes one parameter as input and returns the profile over the prespecified range
###############################################################################

profile_loglike <- function(prof_par, range, no_points, no_est, estpars, Nmif, Np, var.factor){
  
  #  Args:
  #  prof_par: name of the parameter to profile
  #  range: range of values for the profiled parameter
  #  no_points: number of points to test in the search interval for the profiled parameter
  #  no_est: number of estimations per point
  #  estpars: names of the parameters to estimate (including the profiled parameter)
  #  Nmif, Np, var.factor: algorithmic parameters for the iterated filtering algorithm
  
  #  Returns: data frame with the values of the profiled parameter and the profile log likelihood
  
  #  Remove the profiled parameter from the list of estimated parameters
  estpars <- estpars[-which(estpars ==  prof_par)]
  
  #  Search interval for the profiled parameter
  search_interval <- seq(from = range[1], to = range[2], length = no_points)
  
  #  Creates the data frame
  prof <- data.frame(par = rep(search_interval, each = no_est), ll = NA)
  
  #  Mif estimations over the remaining parameters
  rw.sd <- rep(0.05, length(estpars))
  names(rw.sd) <- estpars
  
  mf <- list()
  for(i in 1:nrow(prof)){
    coef(isolmodel, prof_par) <- prof$par[i]  #  Sets the value of the profiled parameter
    theta0 <- coef(isolmodel)
    mf[[i]] <- try(
      mif(isolmodel, Nmif=Nmif, Np=Np, start=theta0,
          rw.sd=rw.sd,
          transform=TRUE, pars=estpars, var.factor=var.factor, ic.lag=20, cooling.factor=0.95,
          method="mif", max.fail=1e2))
    
    if(class(mf[[i]]) == "mif") prof$ll[i] <- logLik(mf[[i]])
  }
  return(prof)
}